-- microYASEP.xx.vhdl
-- créé le 19 février 2012 par Yann Guidon / whygee@f-cpu.org

-- 01 : resynchronisation du reset local  et changement de phase
-- 02 : PC, flags, registres de pipeline & instructions
-- 03 : mise en place des ports et de la RAM instructions
-- 04 : mise en place du banc de registres
--        souci : calculer PC+1 et PC+2 ?
--        solution : fournir juste PC, et NPC en bout
--          de pipeline pour les instructions CALL/CALL2/NPC
-- 05 : PC résolu, CALL possible, ajout chemin d'adresse et de données
-- 06 : ALU/ROP2
-- 07 : manque le chemin de contrôle, conditions, WB_en
-- var01 : changement des ports,
--   R1=flags, est ORé à chaque cycle avec les données entrant sur le port flags_x
--       flag_addr donne le n° du bit à mettre à 1 et flags_in valide l'écriture
--   A1=adresse d'écriture, disponible en sortie sur le port Result (validé par A1_en)
--   D1=donnée à écrire, disponible en sortie sur le port D1
-- piège : WB_en != Wr_en

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
library work;
    use work.yasep_definitions.all;
    use work.yasep_utils.all;

entity microYASEP is
port(

-- La mémoire instruction est en blockRAM, hors du "coeur".
-- elle est lue en continu, la RAM active en permanence
-- donc pas de signal RAM_read_en <= '0';
  RAM_out: in SLV16; -- port de lecture de la RAM instruction
  PC, NPC: inout SLVAI;  -- nouvelle valeur du compteur d'instructions   /!\ LSB=0

-- so we can spy on the core :
  WR_en, WB_en, phase : inout std_logic;
  R1, R2, R3, R4, Result: inout SLVY;
  DST: inout SLV4;

-- GET, PUT :
  GetReadPort: in SLVY;  
  PutWritePort, SRaddress: out SLVY;  
  GET_en, PUT_en: out std_logic;

-- E/S spécifiques
  flag_addr: in SLV4;
  flags_in: in std_logic; -- OR avec R1 à chaque cycle
  A1_en : out std_logic;
  D1: inout SLVY;

  clk, ext_reset : in std_logic);
end microYASEP;

architecture behav of microYASEP is

  signal int_reset, condition, condition2,
           ROP2_X2, Addsub, Carry, Carry_out, compare_signed,
           CondZero : std_logic := '0';
  signal opcode: SLV6 := "000000";
  signal cond_func, form: SLV2 := "00";
  signal Field_SI4, Field_SND : SLV4 := "0000";
  signal SI4, RSI4_out, Reg_SI4, SI4_ext,
         SND, RSND_out,
         ActualA, ActualB, ROP2_RS1, ROP2_RS2,
         ASU, ROP2 : SLVY := (SLVY'range=>'0');
  signal sumAux : unsigned(YASEP_SIZE+1 downto 0);
  signal PC1 : SLVAI := (SLVAI'range=>'0');
  signal int_opcode: integer := 0;

  -- Les autres registres :
  signal  R5, A1, A2, D2, A3, D3,
      A4, D4, A5, D5 : SLVY := (SLVY'range=>'0');

  -- adapte le PC à la largeur des mots
  function SLVAI_to_SLVY(v: SLVAI) return SLVY is
    variable r:SLVY := (SLVY'range=>'0');
  begin
    r(SLVAI'range) := v;
    return r;
  end;

begin

  int_opcode<=safe_to_integer(opcode);

------------------------- Zone asynchrone -------------------------

-- Lecture des registres :

  RSND_out <= SLVAI_to_SLVY(PC) when RAM_out(11 downto 8)=REG_PC
         else R1   when RAM_out(11 downto 8)=REG_R1
         else R2   when RAM_out(11 downto 8)=REG_R2
         else R3   when RAM_out(11 downto 8)=REG_R3
         else R4   when RAM_out(11 downto 8)=REG_R4
         else R5   when RAM_out(11 downto 8)=REG_R5
         else D1   when RAM_out(11 downto 8)=REG_D1
         else A1   when RAM_out(11 downto 8)=REG_A1
         else D2   when RAM_out(11 downto 8)=REG_D2
         else A2   when RAM_out(11 downto 8)=REG_A2
         else D3   when RAM_out(11 downto 8)=REG_D3
         else A3   when RAM_out(11 downto 8)=REG_A3
         else D4   when RAM_out(11 downto 8)=REG_D4
         else A4   when RAM_out(11 downto 8)=REG_A4
         else D5   when RAM_out(11 downto 8)=REG_D5
         else A5;--when RAM_out(11 downto 8)=REG_A5

  RSI4_out <= SLVAI_to_SLVY(PC) when RAM_out(15 downto 12)=REG_PC
         else R1   when RAM_out(15 downto 12)=REG_R1
         else R2   when RAM_out(15 downto 12)=REG_R2
         else R3   when RAM_out(15 downto 12)=REG_R3
         else R4   when RAM_out(15 downto 12)=REG_R4
         else R5   when RAM_out(15 downto 12)=REG_R5
         else D1   when RAM_out(15 downto 12)=REG_D1
         else A1   when RAM_out(15 downto 12)=REG_A1
         else D2   when RAM_out(15 downto 12)=REG_D2
         else A2   when RAM_out(15 downto 12)=REG_A2
         else D3   when RAM_out(15 downto 12)=REG_D3
         else A3   when RAM_out(15 downto 12)=REG_A3
         else D4   when RAM_out(15 downto 12)=REG_D4
         else A4   when RAM_out(15 downto 12)=REG_A4
         else D5   when RAM_out(15 downto 12)=REG_D5
         else A5;--when RAM_out(15 downto 12)=REG_A5

-- chemin de contrôle : valide l'écriture

A1_en <= '1' when WR_en='1' and DST=REG_A1
    else '0';

WR_en <= '1' when WB_en='1' and  FlagNoWriteBack(int_opcode)='0'
    else '0';

WB_en <= '1' when FlagValidInstruction(int_opcode)='1'
      and condition='1'
      and phase='1'
    else '0';

condition <= '0' when FlagNoCondition(int_opcode)='1'
         and (RAM_out(5) xor Condition2)='1'   -- condition is false
         and  form = FORM_EXT -- extended / conditional instruction
   else '1'; -- true otherwise

Condition2 <= R1(safe_to_integer(Field_SI4))  when cond_func="01"
         else  '1'                       when cond_func="00" and Field_SI4=REG_PC
         else Carry                      when cond_func="10" and Field_SI4=REG_PC
         else CondZero                   when cond_func="00"
         else RSI4_out(0)                when cond_func="10"
         else RSI4_out(YASEP_SIZE-1); -- when cond_func="11"  -- toutes les combinaisons ne sont pas couvertes

cond_func <= RAM_out(7 downto 6);
CondZero <= '1' when RSI4_out=(RSI4_out'range=>'0')
       else '0';

-- chemin d'adresse :
  DST <= Field_SND when form(0) = '0' -- Short/Long
    else Field_SI4 when form(1) = '0' -- R/I4
    else RAM_out(R_SND'range); -- correspond à DST3 car c'est le 2è mot

-- mise à jour du compteur de programme
  PC1 <= std_logic_vector(unsigned(PC)+1);
  NPC <= Result(NPC'range) when phase='1' and WR_en='1' and DST=REG_PC
    else SI4(NPC'range) when phase='1' and WB_en='1' and opcode=Op_CALL
    else ASU(NPC'range) when phase='1' and WB_en='1' and opcode=Op_CALL2
    else PC1;

-- /!\ bien faire attention à l'échange qui se produit lors de CALL/CALL2,
--         entre NPC et Result. (ça se passe avec PC1 et SI4/ASU

  Result <= ASU    when opcode(2 downto 0)=GROUP_ASU
       else ROP2   when opcode(2 downto 0)=GROUP_ROP2
       else SLVAI_to_SLVY(PC) when opcode=Op_CALL or opcode=Op_CALL2
       else GetReadPort when opcode=Op_GET
       else SI4;  -- toutes les autres instructions, comme MOV

-- Chemin de données : sélection de SI4
  SI4 <=  RAM_out when form = FORM_RRI16         -- long immédiat
     else SI4_ext when  (form = FORM_RI4)        -- short Imm4
        or (form = FORM_EXT and  RAM_out(1)='1') -- extended
     else Reg_SI4;                               -- short RR

 -- extension de signe
  SI4_ext <= (YASEP_SIZE-5 downto 0 => Field_SI4(3)) & Field_SI4;

----------------- ALU/ROP2 -----------------

--         ROP2_S1  ROP2_S2  ROP2_X2  Addsub  compare_signed
-- ADD        X        X        X       0           0
-- SUB        X        X        X       1           0
-- compare                                (== SUB, only carry-out used)
--    signed  X        X        X       1           1
--  unsigned  X        X        X       1           0
-- AND        1        0        0       0           0
-- ANDN       1        0        0       1           0
-- NAND       1        0        1       0           0
-- OR         X        1        0       0           0
-- ORN        X        1        0       1           0
-- NOR        X        1        1       0           0
-- XOR        0        0        0       0           0
-- XORN       0        0        0       1           0

ROP2_X2 <= '1' when
    opcode=Op_XOR  or
    opcode=Op_XORN
  else '0';

Addsub <= '1' when
    opcode=Op_SUB or
    opcode=Op_CMPS or
    opcode=Op_CMPU or
    opcode=Op_ANDN or
    opcode=Op_ORN or
    opcode=Op_XORN
  else '0';

compare_signed <='1' when
    opcode=Op_CMPS or
    opcode=Op_SMAX
  else '0';

-- XOR préliminaire et ajustement du signe :
ActualB <= ( SND(YASEP_SIZE-1) xor (Addsub and (not compare_signed)) )
         & ( SND(YASEP_SIZE-2 downto 0) xor (YASEP_SIZE-2 downto 0 =>Addsub));

-- XOR the MSB here too for the signed comparisons :
ActualA <= ( SI4(YASEP_SIZE-1) xor compare_signed )
         &   SI4(YASEP_SIZE-2 downto 0);

-- Add/Sub
sumAux <= unsigned('0' & ActualA & '1') + unsigned('0' & ActualB & Addsub);

Carry_out <= std_logic(sumAux(YASEP_SIZE+1));
ASU <= std_logic_vector(sumAux(YASEP_SIZE downto 1));

-- ROP2 :
-- First MUX
ROP2_RS1 <= ActualA and ActualB when
      opcode=Op_AND  or
      opcode=Op_ANDN or
      opcode=Op_NAND
    else    ActualA xor ActualB;

ROP2_RS2 <= ActualA  or ActualB when
         opcode=Op_OR  or
         opcode=Op_ORN or
         opcode=Op_NOR
       else ROP2_RS1;

-- last XOR
ROP2 <= ROP2_RS2 xor (ROP2'range => ROP2_X2);


------------------------- Zone synchrone -------------------------

  process (clk) begin
    -- resynchronisation du reset local
    if rising_edge(clk) then
      int_reset <= ext_reset;
    end if;
  end process;

  process (clk, int_reset)
  begin
    if int_reset='0' then
      phase <='1';
      PC <= (PC'range => '1'); -- provoque un rebouclage à 0 dans PC1
      -- met l'instruction "NOP" dans les registres concernés :
      opcode <= "000000";
      form   <= "00";
      Field_SND <= "0000";
      Field_SI4 <= "0000";
    else
      if rising_edge(clk) then

        -- mise à jour des variations
        if flags_in='1' then
          R1(safe_to_integer(flag_addr)) <= '1';
        end if;

        -- changement de phase
        if phase='0' then
-- phase "fetch" : on écrit la sortie de la RAM dans des registres temporaires
          phase <= '1';
              form  <= RAM_out(1 downto 0);
             opcode <= RAM_out(R_OPCODE'range);
          Field_SND <= RAM_out(R_SND'range);
              SND   <= RSND_out;

          -- gère la longueur de l'instruction
          if RAM_out(0)='1' then
            -- instruction longue : Lire la suite
            PC <= NPC;
            Field_SI4 <= RAM_out(R_SI4'range); -- destination pour IMM16, source pour EXT
            if RAM_out(1)='1' then  -- latche si EXT
              Reg_SI4 <= RSI4_out;  -- champ inutilité en forme IMM16
            end if;
          else
            -- instruction courte : Faire une pause d'une phase
            if RAM_out(1)='1' then -- ne latche que le nécessaire
              Field_SI4 <= RAM_out(R_SI4'range); -- FORM_RI4
            else
              Reg_SI4 <= RSI4_out;   -- FORM_RR
            end if;
          end if;

        else
-- phase "execute" : on calcule et latche des valeurs, adresses et conditions
          phase <= '0';

          if WB_en='1' and FlagChangeCarry(int_opcode)='1' then
              Carry <= Carry_out;
          end if;
          if WR_en='1' then
            case DST is
              when REG_R1 => R1 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to R1";
              when REG_R2 => R2 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to R2";
              when REG_R3 => R3 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to R3";
              when REG_R4 => R4 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to R4";
              when REG_R5 => R5 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to R5";
              when REG_A1 => A1 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to A1";
              when REG_D1 => D1 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to D1";
              when REG_A2 => A2 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to A2";
              when REG_D2 => D2 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to D2";
              when REG_A3 => A3 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to A3";
              when REG_D3 => D3 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to D3";
              when REG_A4 => A4 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to A4";
              when REG_D4 => D4 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to D4";
              when REG_A5 => A5 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to A5";
              when REG_D5 => D5 <= Result;
 report "writing " & SLV_to_Hex(Result) &  " to D5";
              when others => -- NPC et PC sont gérés autre part
 report "writing " & SLV_to_Hex(Result) &  " to PC";
            end case;
          end if;
          PC <= NPC;
        end if;
      end if;
    end if;
  end process;

end behav;
