-- test_Y.vhd
-- created mar. févr. 21 14:26:03 CET 2012 by whygee@f-cpu.org

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
library work;
    use work.yasep_definitions.all;
    use work.yasep_utils.all;

entity test_uY is
  -- empty
end test_uY;

architecture toplevel of test_uY is
  signal WR_en  : std_logic := '0';
  signal WB_en  : std_logic := '0';
  signal phase  : std_logic := '0';
  signal clk    : std_logic := '0';
  signal reset  : std_logic := '0';

  signal PC : SLVAI := (SLVAI'range => '0');
  signal NPC : SLVAI := (SLVAI'range => '0');
  signal RAM_out : SLV16 := (SLV16'range => '0');
  signal DST : SLV4 := "0000";

  signal GetReadPort, PutWritePort, SRaddress: SLVY := (SLVY'range => '0');
  signal GET_en  : std_logic := '0';
  signal PUT_en  : std_logic := '0';

  signal flag_addr : SLV4 := "0000";
  signal flags_in  : std_logic := '0';
  signal A1_en : std_logic := '0';
  signal R1, R2, R3, R4, D1, Result: SLVY := (SLVY'range => '0');

begin
  ProgRAM:
    entity work.SynchRam16
    port map(
      clk => clk,
      reset => reset,
      NPC => NPC,
      RAM_out => RAM_out);

  uY:
    entity work.microYASEP
    port map(
      NPC => NPC,
      PC => PC,
      RAM_out => RAM_out,
      DST => DST,
      phase => phase,
      WB_en => WB_en,
      WR_en => WR_en,
      R1 => R1,
      R2 => R2,
      R3 => R3,
      R4 => R4,

      GetReadPort => GetReadPort,
      PutWritePort =>  PutWritePort,
      SRaddress => SRaddress,
      GET_en => GET_en,
      PUT_en => PUT_en,

      flag_addr => flag_addr,
      flags_in  => flags_in,
      A1_en => A1_en,
      D1 => D1,
      Result => Result,

      clk => clk,
      ext_reset => reset);

  observe: process(clk) begin
    if clk='1' then
      report " phase="     & std_logic'image(phase)
           & " WR_en="     & std_logic'image(WR_en)
           & " WB_en="     & std_logic'image(WB_en)
--           & " GET_en="    & std_logic'image(GET_en)
--           & " PUT_en="    & std_logic'image(PUT_en)
--           & " A1_en="     & std_logic'image(A1_en)
         & "   PC="        & SLV_to_Hex(PC & '0')
          & "  NPC="       & SLV_to_Hex(NPC & '0')
          & "  RAM_out="   & SLV_to_Hex(RAM_out)
          & "  Result="    & SLV_to_Hex(Result)
          & " DST="       & SLV4_to_Hex(DST)
         & "  R1="        & SLV_to_Hex(R1)
          & " R2="        & SLV_to_Hex(R2)
          & " R3="        & SLV_to_Hex(R3)
          & " R4="        & SLV_to_Hex(R4)
--          & "  SRaddress=" & SLV_to_Hex(SRaddress)
--          & "  D1="        & SLV_to_Hex(D1)
--         & " "         & integer'image(safe_to_integer())
      ;
    end if;
  end process;

  bench: process
    variable i: integer := 0;
  begin
                wait for 1 us;
    clk <= '1'; wait for 1 us;
    clk <= '0'; wait for 1 us;
    clk <= '1'; wait for 1 us;
    clk <= '0'; wait for 1 us;

    report " *** releasing reset *** ";
    reset <= '1'; wait for 1 us;

    while i < 50 loop
      clk <= '1'; wait for 1 us;
      clk <= '0'; wait for 1 us;
      i:=i+1;
    end loop;

    wait;
  end process;

end toplevel;
