-- yasep_utils.vhdl
-- created mar. févr. 21 18:10:32 CET 2012 by whygee@f-cpu.org
-- OK it's a small convenience package to ease simulation. No big deal here. 

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
library work;
    use work.yasep_definitions.all;

package yasep_utils is
  function safe_to_integer(u:std_logic_vector) return integer;
  function SLV4_to_Hex(s:SLV4) return character;
  function SLV_to_Hex(s:std_logic_vector) return string;
end yasep_utils;

package body yasep_utils is

  function safe_to_integer(u:std_logic_vector) return integer is
  begin
   if Is_X(u) then
      return 0;
    else
      return to_integer(unsigned(u));
    end if;
  end;

  function SLV4_to_Hex(s:SLV4) return character is
  begin
    case s is
      when "0000" => return '0';
      when "0001" => return '1';
      when "0010" => return '2';
      when "0011" => return '3';
      when "0100" => return '4';
      when "0101" => return '5';
      when "0110" => return '6';
      when "0111" => return '7';
      when "1000" => return '8';
      when "1001" => return '9';
      when "1010" => return 'A';
      when "1011" => return 'B';
      when "1100" => return 'C';
      when "1101" => return 'D';
      when "1110" => return 'E';
      when "1111" => return 'F';
      when others => return '?';
    end case;
  end;

  function SLV_to_Hex(s:std_logic_vector) return string is
    variable u:std_logic_vector(s'length+3 downto 0);
    variable t: string((s'length-1)/4 downto 0);
    variable i: integer:= 0;
    variable j: integer:= 0;
  begin
    u:= "0000" & s;

    while (i < s'length) loop
      t(j) := SLV4_to_Hex(u(i+3 downto i));
      i := i+4;
      j := j+1;
    end loop;
    return t;
  end;

end yasep_utils;
